#include "alg/color.hpp"
#include <math.h>

Color::Color() 
{
  set(0.0, 0.0, 0.0, 0.0);
}

Color::Color(const Color &c)
{
  set(c.colors[RedChannel], 
      c.colors[GreenChannel], 
      c.colors[BlueChannel], 
      c.colors[AlphaChannel]);
}

Color::Color(float red, float green, float blue, float alpha)
{
  set(red, green, blue, alpha);
}

void Color::set(float r, float g, float b, float a)
{
  colors[RedChannel]   = r;
  colors[GreenChannel] = g;
  colors[BlueChannel]  = b;
  colors[AlphaChannel] = a;
}

const float *const Color::get() const
{
  return colors;
}

void Color::setR(float r)
{
  colors[RedChannel]   = r;
}

float Color::r() const
{
  return colors[RedChannel];
}

void Color::setG(float g)
{
  colors[GreenChannel]   = g;
}

float Color::g() const
{
  return colors[GreenChannel];
}

void Color::setB(float b)
{
  colors[BlueChannel]   = b;
}

float Color::b() const
{
  return colors[BlueChannel];
}

void Color::setA(float a)
{
  colors[AlphaChannel]   = a;
}

float Color::a() const
{
  return colors[AlphaChannel];
}

static bool close(float l, float r, float t=1e-6)
{
  float difference = fabsf(l-r);
  l = fabsf(l);
  r = fabsf(r);
   
  return (difference <= (t*l) or difference <= (t*r));
}
bool operator==(const Color &x, const Color &y)
{
  return (close(x.colors[Color::RedChannel]  ,y.colors[Color::RedChannel]) and
         close(x.colors[Color::GreenChannel],y.colors[Color::GreenChannel]) and 
         close(x.colors[Color::BlueChannel] ,y.colors[Color::BlueChannel]) and
         close(x.colors[Color::AlphaChannel],y.colors[Color::AlphaChannel]));
}

std::ostream &operator<<(std::ostream &out, const Color &c)
{
  out << c.r() << " " << c.g() << " " << c.b() << " " << c.a();
  return out;
}
