#ifndef _GLCOLOR_H_
#define _GLCOLOR_H_

#include <boost/serialization/export.hpp>
#include <boost/serialization/nvp.hpp>
#include <boost/serialization/version.hpp>

#include <ostream>

//! convienece class for handling opengl colors
class Color
{
public:
  Color();
  Color(const Color &);
  //! initialize with red, green, blue, alpha
  Color(float r, float g, float b, float a=1.0);

  //! set all channels simultaneously 
  void set(float r, float g, float b, float a=1.0);
  //! return an array of 4 colors (stored by class)
  const float* const get() const;
  void setR(float);
  float r() const;
  void setG(float);
  float g() const;
  void setB(float);
  float b() const;
  void setA(float);
  float a() const;

  enum color_channels { RedChannel, GreenChannel, BlueChannel, AlphaChannel };
  friend bool operator==(const Color&, const Color&);
  friend std::ostream& operator<<(std::ostream&, const Color&);

protected:
  float colors[4];

private:
  friend class boost::serialization::access;
  template<class Archive>
  void serialize(Archive& ar, const unsigned int /*version*/) {
    ar & boost::serialization::make_nvp("red",   colors[RedChannel]);
    ar & boost::serialization::make_nvp("green", colors[GreenChannel]);
    ar & boost::serialization::make_nvp("blue",  colors[BlueChannel]);
    ar & boost::serialization::make_nvp("alpha", colors[AlphaChannel]);
  }
};
BOOST_CLASS_EXPORT(Color)

#endif

