#include "qui/motif_editor/MotifEditor.hpp"
#include "alg/sequence.hpp"

#include <set>
#include <string>

using namespace std;

MotifEditor::MotifEditor(MussaRef m, QWidget *parent)
  : QWidget(parent),
    analysis(m),
    editor_layout(0),
    button_layout(0),
    table(0),
    delegate(0),
    applyButton(0),
    model(0)
{
  setupWidgets();
  
  assert (m != 0);
  const set<Sequence> &motif = analysis->motifs();
  vector<Sequence> motif_seq(motif.begin(), motif.end());

  applyButton->setFocusPolicy(Qt::StrongFocus);
  connect(applyButton, SIGNAL(clicked()), this, SLOT(updateAnalysisMotifs()));
  button_layout->addStretch();
  button_layout->addWidget(applyButton);
  
  table->setItemDelegate(delegate);

  editor_layout->addWidget(table);
  editor_layout->addLayout(button_layout);
  setLayout(editor_layout);
  
  updateTitle();
  updateModel();
}

void MotifEditor::setupWidgets()
{
  editor_layout = new QVBoxLayout(this);
  button_layout = new QHBoxLayout(this);
  table = new QTableView(this);
  delegate = new MotifEditorDelegate(this);
  applyButton = new QPushButton("apply");
}

void MotifEditor::updateModel()
{
  MotifModel *new_model = new MotifModel(analysis);
   
  // if there was an old model, delete it
  if (model) {
    delete model;
  }
  // update the QTableView
  model = new_model;  
  table->setModel(model);
  updateView();
}

void MotifEditor::updateAnalysisMotifs()
{
  // This function is _sooo_ not thread safe
  // erase motifs
  Color motif_default = analysis->colorMapper()->typeColor("motif");
  analysis->colorMapper()->erase("motif");
  analysis->colorMapper()->appendTypeColor("motif", motif_default);

  // add our motifs back
  vector<Sequence> new_motifs;
  vector<Color> new_colors;

  for(MotifModel::iterator md_i =  model->begin();
      md_i != model->end();
      ++md_i)
  {
    if (md_i->getSequence().size() > 0 && md_i->isEnabled()) {
      new_motifs.push_back(md_i->getSequence());
      new_colors.push_back(md_i->getColor());
    }
  }
  analysis->set_motifs(new_motifs, new_colors);

  emit changedMotifs();
}

void MotifEditor::updateTitle() 
{
  std::string title("Motif Editor: ");
  if (analysis) {
    title += analysis->get_title();
  }
  setWindowTitle(title.c_str());
}

void MotifEditor::updateView()
{
   for (int row = 0; row < model->rowCount(QModelIndex()); ++row)
       table->resizeRowToContents(row);
   for (int column = 0; column < model->columnCount(QModelIndex()); ++column)
       table->resizeColumnToContents(column);
}


